#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
*****************************************************************************************************
working for Python 2.7

LIRIS Canvas to file printer : this module takes a canvas from Tkinter and manage to export it into 
.png or .pdf files. this module require Tkinter, ReportLab and Image (PIL) modules to be installed.

this module will generate exception on errors with the type CanException, problem will be described
in CenException.cause.

User functions : 

  engine.save_PNG : export the canvas to a .png file
  engine.save_PDF : export the canvas to a .pdf file

Author : Tristan Delizy, 

*****************************************************************************************************
"""


#====================================================================================================#
#IMPORTS
#====================================================================================================#
try : 
  imports = "Tkinter"
  from Tkinter import *
  imports = "Image, ImageTk, ImageFile, ImageDraw"
  import Image, ImageTk, ImageFile, ImageDraw, ImageFont
  imports = "os, sys, math"
  import os, sys, math
  imports = "zlib"
  import zlib
except : 
  print "error : ", imports
  e = "CanExp ERROR : crash on imports : unable to load one of the modules ("+imports+")."
  raise CanException(e)

#====================================================================================================CanException
#class of exception to get differrent comportement for error treatments
class CanException(Exception):
    def __init__(self,cause):
        self.cause = cause
    
    def __str__(self):
        return self.cause

#====================================================================================================Engine
#engine of the printer 
class Engine():
#----------------------------------------------------------------------------------------------------__init__
#the constructor of the printer engine. Takes a tkinter canvas reference as param.
#you can also give the output size in option.
  def __init__(self, canvas):
    if not isinstance(canvas, Canvas):
      e = "CanExp ERROR : the \'canvas\' parameter seems not to be a Tkinter Canvas."
      raise CanException(e)
    self.can = canvas
    self.ratio = float(canvas.winfo_height())/float(canvas.winfo_width())
 
    ##this modifications of scale and position will be effective on the canvas displayed but will be canceled 
    ##after the export.*
    self.can.scale("all", 0, 0,700.0/float(canvas.winfo_width()), (700*self.ratio)/float(canvas.winfo_height()))
    self.img = Image.new("RGB", (700, int(700*self.ratio)), self.can.cget("bg"))
    self.draw = ImageDraw.Draw(self.img)
    self.font = ImageFont.load_default()

#----------------------------------------------------------------------------------------------------buildImg
#Method analysing the canvas to export and regenerate it with Image.draw lib.
#object implementeds are the following (non implemented objects are ignored): 

#	rectangle with options {fill, outline}
#	text with options {fill, width}, only if width = 1 (vertical text in tkinter, rotated text with Image)
#	line with options {fill, width, dash}

  def buildImg(self):
    ObjectList = self.can.find_all()
    for i in range(len(ObjectList)):
      #rectangle
      if self.can.type(ObjectList[i]) == "rectangle":
	getCoords = self.can.coords(ObjectList[i])
	getFill = self.can.itemcget(ObjectList[i], "fill")
	getOutline = self.can.itemcget(ObjectList[i], "outline")
	self.draw.rectangle(getCoords, fill=getFill, outline=getOutline)
      #line
      elif self.can.type(ObjectList[i]) == "line":
	getCoords = self.can.coords(ObjectList[i])
	getFill = self.can.itemcget(ObjectList[i], "fill")
	getWidth = self.can.itemcget(ObjectList[i], "width")
	getWidth = int(float(getWidth))
	getDash = self.can.itemcget(ObjectList[i], "dash")
	if getDash != "":
	  dash = int(getDash[0]) + int(getDash[2])
	  seg = int(math.sqrt((getCoords[2]-getCoords[0])*(getCoords[2]-getCoords[0])+(getCoords[3]-getCoords[1])*(getCoords[3]-getCoords[1])))
	  count = seg/dash
	  for i in range(int(count*2-1)):
	    if i % 2 == 0:
	      x0=getCoords[0]+(int(getCoords[2]-getCoords[0])*i/float(count*2))
	      y0=getCoords[1]+(int(getCoords[3]-getCoords[1])*i/float(count*2))
	      x1=getCoords[0]+(int(getCoords[2]-getCoords[0])*(i+1)/float(count*2))
	      y1=getCoords[1]+(int(getCoords[3]-getCoords[1])*(i+1)/float(count*2))
	      self.draw.line([x0,y0,x1,y1],fill=getFill, width=getWidth)
	else:  
	  self.draw.line(getCoords,fill=getFill, width=getWidth)
      #text
      elif self.can.type(ObjectList[i]) == "text":
	getCoords = self.can.coords(ObjectList[i])
	getText = self.can.itemcget(ObjectList[i], "text")
	getFill = self.can.itemcget(ObjectList[i], "fill")
	getWidth = self.can.itemcget(ObjectList[i], "width")
	getAnchor = self.can.itemcget(ObjectList[i], "anchor")
	if getAnchor == "w":
	  getCoords[0] = getCoords[0]+int(self.font.getsize(getText)[0]/2.0)
	if len(getText.split("\n")) > 1:
	  for i in range(len(getText.split("\n"))):
	    self.draw.text((getCoords[0]-int(self.font.getsize(getText)[0]/(2.0*len(getText.split("\n")))),getCoords[1]-int(self.font.getsize(getText)[1]*(len(getText.split("\n"))-1-i)/float(len(getText.split("\n"))-1))),text=getText.split("\n")[i], fill=getFill)
	elif int(getWidth) == 1:
	  img_txt = Image.new('RGB', self.font.getsize(getText), self.can.cget("bg"))
	  draw_txt = ImageDraw.Draw(img_txt)
	  draw_txt.text((0,0), getText, font=self.font, fill=getFill)
	  x, y = img_txt.size
	  self.img.paste(img_txt.rotate(90), (int(getCoords[0]),int(getCoords[1])))
	else:
	  self.draw.text((getCoords[0]-int(self.font.getsize(getText)[0]/2.0),getCoords[1]-int(self.font.getsize(getText)[1]/2.0)),text=getText, fill=getFill)
	  
#----------------------------------------------------------------------------------------------------save_PNG
#Method saving the canvas translated to Image.draw into .png file    
  def save_PNG(self, file_name):
    if type(file_name)==str and file_name != "":
      self.buildImg()
      try:
	img2 = self.img.crop((20,35,685,510))
	img2.save(file_name+".png")
	#self.img.save(file_name+".png")
      except:
	e = "CanExp ERROR : unable to save the file",file_name+".png."
	self.rescale()
	raise CanException(e)
      self.rescale()
      
#----------------------------------------------------------------------------------------------------save_PDF
#Method saving the canvas translated to Image.draw into .pdf file    
  def save_PDF(self, file_name):
    if type(file_name)==str and file_name != "":
      self.buildImg()
      try:
	self.img.save(file_name+".pdf")
      except:
	e = "CanExp ERROR : unable to save the file",file_name+".pdf."
	self.rescale()
	raise CanException(e)
      self.rescale()
      
#----------------------------------------------------------------------------------------------------rescale
#Method restoring the original look and fill of the Tkinter canvas     
  def rescale(self):
    self.can.scale("all",0,0,float(self.can.winfo_width())/700, float(self.can.winfo_height())/(700*self.ratio))
  