#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
*****************************************************************************************************
LIRIS Evaluator of human activity recognition and localisation algorithms - harl
GUI application (use : harlcore.py)

Author : Tristan Delizy,
*****************************************************************************************************
"""
"""
*****************************************************************************************************
to get more info about what is done in there go to the file "harlcore.py"

*****************************************************************************************************
"""

#====================================================================================================#
#IMPORTS
#====================================================================================================#
from Tkinter import *
import tkColorChooser, tkFileDialog
import  tkMessageBox
import os, sys, glob, copy
from time import localtime
import xml.etree.ElementTree as xml
import harlcore as hc
import xmlDisp as xDisp
import actreader as AcR

#Image (PIL) lib is needed for export in .png / .pdf but the app run without it, if loading fail, these options will be disabled.
try :
  import CanExp as ce
  import Image, ImageTk
except :
  IMAGE_LIB = 0
else :
  IMAGE_LIB = 1


#====================================================================================================#
#CONSTANTS
#====================================================================================================#
DEFAULT_THRESHOLDS_VALUE=0.5
THRESHOLDS=["trt","tpt", "srt", "spt"]
CURVES=["recall", "precision", "fscore"]
DEFAULT_CURVE_COLORS=['#e00','#0e0','#00e']
ACTION_CLASSES=["1  DI", "2  GI", "3  BO", "4  EN", "5  ET", "6  LO", "7  UB", "8  HS", "9  KB", "10  TE"]

#====================================================================================================HarlGUI
#Main class of the GUI application
class HarlGUI(Frame):
#----------------------------------------------------------------------------------------------------__init__
#constructor
  def __init__(self, parent, aCurPath):
    Frame.__init__(self, parent, class_="HarlGUI")
    self.grid(sticky=N+S+E+W)
    self.top=self.winfo_toplevel()
    self.parent = parent
    self.curPath = aCurPath
    self.evaluator = hc.Evaluator("gui")
    self.initUI()

#----------------------------------------------------------------------------------------------------initUI
#initialisation of the main frame
  def initUI(self):
    #Layouts managment
    self.top.rowconfigure(1,weight=1)
    self.top.columnconfigure(1,weight=1)
    self.top.geometry("800x600+0+0")
    self.top.aspect(6,5,6,5)
    self.top.minsize(width=800, height=800)
    self.top.title("Harl - GUI")
    self.top.rowconfigure(0, minsize=10)
    self.top.rowconfigure(3, minsize=10)
    self.top.columnconfigure(3, minsize=10)
    self.top.columnconfigure(0, minsize=10)
    self.spacer01=Frame(self.parent)
    self.spacer02=Frame(self.parent)
    self.spacer03=Frame(self.parent)
    self.spacer04=Frame(self.parent)
    self.spacer01.grid(row=0,column=0,rowspan=3,sticky=N+S+E+W)
    self.spacer02.grid(row=0,column=3,rowspan=3,sticky=N+S+E+W)
    self.spacer03.grid(row=3,column=1,columnspan=2,sticky=N+S+E+W)
    self.spacer03.grid(row=0,column=1,columnspan=2,sticky=N+S+E+W)

    #diagrams panel
    self.CtrlDiag = HarlCtrlDiagramGUI( self)
    self.CtrlDiag.grid(row=1,column=1,sticky=S+N+E+W)

    #main control panel
    self.CtrlMain = HarlCtrlMainGUI(self)
    self.CtrlMain.grid(row=2,column=1,sticky=S+N+E+W)

    #file control panel
    self.CtrlFile = HarlCtrlFileGUI(self)
    self.CtrlFile.grid(row=0,column=2,rowspan=3,sticky=S+N+E+W)

    #Menus managment - GENERAL-
    self.menuBar = Menu(self.top)
    self.top["menu"] = self.menuBar
    self.subMenu_Files = Menu(self.menuBar, tearoff=0)
    self.subMenu_Edition = Menu(self.menuBar, tearoff=0)
    self.subMenu_Help = Menu(self.menuBar, tearoff=0)
    self.menuBar.add_cascade(label="Files", menu=self.subMenu_Files)
    self.menuBar.add_cascade(label="Edition", menu=self.subMenu_Edition)
    self.menuBar.add_cascade(label="Help", menu=self.subMenu_Help)

    #Menus managment - Help menu
    self.subMenu_Help.add_command(label="Manual", command=self._Manual)
    self.subMenu_Help.add_command(label="About", command=self._About)

    #Menus managment - Files menu
    if IMAGE_LIB == 1:
      self.subMenu_Files.add_command(label="Export in .png", command=self._ExPNG)
      self.subMenu_Files.add_command(label="Export in .pdf", command=self._ExPDF)
    else :
      self.subMenu_Files.add_command(label="Export in .png", command=self._ExPNG, state = "disable")
      self.subMenu_Files.add_command(label="Export in .pdf", command=self._ExPDF, state = "disable")
    self.subMenu_Files.add_command(label="Export in .txt", command=self._ExTXT)
    self.subMenu_Files.add_separator()
    self.subMenu_Files.add_command(label="Quit", command=self._Quit)

    #Menus managment - Edition menutestRendering()
    self.subMenu_Edition.add_command(label="Reset files", command=self.CtrlFile._ResetFiles)
    self.subMenu_Edition.add_separator()
    self.subMenu_Edition.add_radiobutton(value="no scale", label="no scale", variable=self.CtrlDiag.GetGradStrigVar(),command=self.CtrlDiag.Draw_graduation)
    self.subMenu_Edition.add_radiobutton(value="light scale",label="light scale", variable=self.CtrlDiag.GetGradStrigVar(),command=self.CtrlDiag.Draw_graduation)
    self.subMenu_Edition.add_radiobutton(value="accurate scale", label="accurate scale", variable=self.CtrlDiag.GetGradStrigVar(),command=self.CtrlDiag.Draw_graduation)

    #Binding - curves control panel
    self.CtrlMain.buttonColorTr.bind("<Button-1>",self._ChangeCurveColor_RECALL)
    self.CtrlMain.buttonColorTp.bind("<Button-1>",self._ChangeCurveColor_PRECISION)
    self.CtrlMain.buttonColorFsc.bind("<Button-1>",self._ChangeCurveColor_FSCORE)
    self.CtrlMain.spinSample.bind("<<samples>>", self._SetSampling)

    #Binding - files control pannel
    self.CtrlFile.buttonProcess.bind("<Button-1>",self._Process)

    #drawing the canvas
    self.CtrlDiag.Draw()

    #informing of missing libs if any
    if IMAGE_LIB != 1:
      text = "This program uses the Image and ImageTk for the module CanExp to export graphs, but it seem's that one or more"
      text += " of these libs are missing. The application will run normally but the export function will be disabled."
      tkMessageBox.showerror("lib import failure", text)

#----------------------------------------------------------------------------------------------------ChangeCurveColor
#asking the user a new color for a curve throught a TkColorChooser
  def ChangeCurveColor(self, curve, button):
    ColorPopUp = tkColorChooser.Chooser()
    result=ColorPopUp.show(initialcolor=self.CtrlDiag.curveColors[curve], parent=self.CtrlFile.buttonGDisp)
    if result == None or result[1] == None:
      return
    self.CtrlDiag.curveColors[curve]=result[1]
    button.config(bg=self.CtrlDiag.curveColors[curve], activebackground=self.CtrlDiag.curveColors[curve])
    self.CtrlDiag.Draw()

#====================================================================================================#
#HANDLERS
#declaring handler with optionnal events make them usable for handling events and as command for widgets
#====================================================================================================#
#----------------------------------------------------------------------------------------------------_Quit
  def _Quit(self, event = None):
    sys.exit(1)

#----------------------------------------------------------------------------------------------------_About
  def _About(self, event = None):
    text = "http://liris.cnrs.fr/harl2012/"
    tkMessageBox.showinfo("About",text)

#----------------------------------------------------------------------------------------------------_Manual
  def _Manual(self, event = None):
    text = "Importing files\n",\
    "To import xml files in the application use the add button below the section where you want to add files (detection or ground truth). ",\
    "Only .xml files  can be loaded you can remove anyone of them from the list by selecting it and clicking on the \"delete\" button. The ",\
    "files list can be reset by using the \"reset\" button. To see the content of any file select it and click on the \"display\" button below the list.\n\n",\
    "Main Processing\n",\
    "In order to launch the scoring calculation, you must first import some xml files in each dataset. The 4 sliders under the graphic zone determine ",\
    "the fix values for each thresholds. Radio buttons are used to select the variant threshold and the number of calculated point for each curve ",\
    "can be set on the samples count spin box.\n ",\
    "When all settings seems fine you can press the \"Process\" button, it may be long particularly if you process a high count of files or samples.\n\n",\
    "Other functions\n",\
    "To get the confusion matrix for the dataset click the \"confusion matrix\" button. To get back in curve mode click the \"curves\" button.",\
    "In order to get a preview of video and time line matching visualization use the selector to choose a video and click on \"preview\"",\
    "You can have a look on xml files by selecting one and clicking on \"display\"\n\n",\
    "Exporting files\n",\
    "the application can export results in three different format. To export curves and/or matrix use the file ? export in .png/.pdf menu. ",\
    "These options will be disabled if you don't have the following libraries : Image, ImageTk, ImageFile, ImageDraw. It will save a file in ",\
    "the asked format in ./output."
    tkMessageBox.showinfo("Manual", text)

#----------------------------------------------------------------------------------------------------_ExPNG
#export diagram in .png : see CanExp.py for détails (imports, exceptions, ...)
  def _ExPNG(self, event = None):
    outDir = os.getcwd()+"/output"
    if not os.path.isdir(outDir):
      os.makedirs(outDir)
    name = outDir+"/"+str(localtime()[0])+"-"+str(localtime()[1])+"-"+str(localtime()[2])+"_"+str(localtime()[3])+"h"+str(localtime()[4])+"m"+str(localtime()[5])
    try:
      printer = ce.Engine(self.CtrlDiag.canvas)
      printer.save_PNG(name)
    except ce.CanException as e :
      tkMessageBox.showerror("error", e.cause)

#----------------------------------------------------------------------------------------------------_ExPDF
#export diagram in .pdf : see CanExp.py for détails (imports, exceptions, ...)
  def _ExPDF(self, event = None):
    outDir = os.getcwd()+"/output"
    if not os.path.isdir(outDir):
      os.makedirs(outDir)
    name = outDir+"/"+str(localtime()[0])+"-"+str(localtime()[1])+"-"+str(localtime()[2])+"_"+str(localtime()[3])+"h"+str(localtime()[4])+"m"+str(localtime()[5])
    try:
      self.CtrlDiag.Draw_onlyDiag()
      printer = ce.Engine(self.CtrlDiag.canvas)
      printer.save_PDF(name)
      self.CtrlDiag.Draw()
    except ce.CanException as e :
      tkMessageBox.showerror("error", e.cause)

#----------------------------------------------------------------------------------------------------_ExTXT
#export diagram in .txt
  def _ExTXT(self, event = None):
    if self.CtrlDiag.GetDiagOnScreen() == None :
      text = "there is no values for export."
      tkMessageBox.showerror("export error", text)
      return
    #precision of the numbers (for exemple p=3 will crash results as "0.xxx" )
    p = 3

    #directory and file creation
    outDir = os.getcwd()+"/output"
    if not os.path.isdir(outDir):
      os.makedirs(outDir)
    text_file = open(outDir+"/"+str(localtime()[0])+"-"+str(localtime()[1])+"-"+str(localtime()[2])+"_"+str(localtime()[3])+"h"+str(localtime()[4])+"m"+str(localtime()[5])+".txt", 'w')

    #Curves values
    #header creation, for n thresholds(Ti) and m measures(Mi), it'll look like this :
    #T1    T2    ...Tn    M1    M2    ...Mm spacing is adjusted by precision
    header = "##Curves\n# "
    for i in range(len(THRESHOLDS)):
      header += THRESHOLDS[i][0:3].ljust(p+3)
    for i in range(len(CURVES)):
      header += CURVES[i][0:3].ljust(p+3)
    header += "\n"
    text_file.write(header)
    #crashing datas,
    s = self.CtrlMain.GetSamplesCount()
    for j in range(int(s)):
      line = "  "
      for i in range(len(THRESHOLDS)):
	if THRESHOLDS[i] != self.CtrlMain.SelectedThreshold:
	  t = str(self.CtrlMain.GetFixValue(THRESHOLDS[i]))
	else:
	  t = str(float(j)/float(s))
	line += t[0:p+2].ljust(p+3)
      for i in range(len(CURVES)):
	c = str(self.CtrlDiag.diagrams[self.CtrlDiag.GetDiagOnScreen()].GetCurves()[CURVES[i]].GetValue(j))
	line += c[0:p+2].ljust(p+3)
      line += "\n"
      text_file.write(line)


    #Fixed performance
    #header creation, for n thresholds(Ti) and m measures(Mi)
    text_file.write("\n")
    header = "##Fixed performance\n# "
    for i in range(len(THRESHOLDS)):
      header += THRESHOLDS[i][0:3].ljust(p+4)
    for i in range(len(CURVES)):
      header += CURVES[i][0:3].ljust(p+4)
    header += "\n"
    text_file.write(header)
    #crashing datas.
    line = "  "
    for i in range(len(THRESHOLDS)):
      t = str(self.CtrlMain.GetFixValue(THRESHOLDS[i]))
      line += t[0:p+2].ljust(p+4)
    for i in range(len(self.CtrlDiag.PV_results)):
      t = str(self.CtrlDiag.PV_results[i])
      line += t[0:p+2].ljust(p+4)
    line += "\n"
    text_file.write(line)

    #Integrated performance
    #header creation, for n thresholds(Ti)
    text_file.write("\n")
    header = "##Integrated performance\n# "
    for i in range(len(THRESHOLDS)):
      header += THRESHOLDS[i][0:3].ljust(p+4)
    header += "\n"
    text_file.write(header)
    #crashing datas.
    line = "  "
    for i in range(len(self.CtrlDiag.Fsc_results)):
      t = str(self.CtrlDiag.Fsc_results[i])
      line += t[0:p+2].ljust(p+4)
    line += "\n"
    text_file.write(line)

    #Combined performance
    #header creation
    text_file.write("\n")
    text_file.write("##Combined performance\n")
    #crashing data.
    line = "  "
    t = str(self.CtrlDiag.combined_perf)
    line += t[0:p+2].ljust(p+4)
    line += "\n"
    text_file.write(line)

    text_file.close()

#----------------------------------------------------------------------------------------------------_Preview
#launch an additional windows to give a preview of the video and annotation
  def _Preview(self, event = None):
    if self.CtrlFile.optionMenuPreview_var.get() == "" :
        tkMessageBox.showinfo("Preview", "please select a video before launching the preview")
        return
    else:
	try:
	  TimeLineWindow = Toplevel()
	  TimeLineWindow.geometry("1000x200+100+400")
	  a=VideoTimeLiner(TimeLineWindow, self.evaluator, self.evaluator.inputDataSet.getVideoByName(self.CtrlFile.optionMenuPreview_var.get()), self.evaluator.groundTruthDataSet.getVideoByName(self.CtrlFile.optionMenuPreview_var.get()))
	  a.grid()
	except hc.harlException as e :
          tkMessageBox.showerror("error", e.cause)

        self.dlg = Toplevel()
        self.dlg.geometry("300x200+100+100")
        self.XmlToPreview = "EMPTY"
        self.dlgLabelXml = Label(self.dlg, text="Fichier XML :")
        self.dlgLabelImg = Label(self.dlg, text="Préfixe :")
        self.dlgImgEntryVar = StringVar()
        self.dlgImgEntry = Entry(self.dlg, textvariable=self.dlgImgEntryVar)
        self.dlgXmlButton = Button(self.dlg, text="choose", command = self._GetXml)
        self.dlgOk = Button(self.dlg, text="ok", command=self._GetImgPath)
        self.dlgCancel = Button(self.dlg, text="cancel", command=self._CancelPreview)
        self.dlgLabelXml.grid(row=0,column=0,sticky=E+W)
        self.dlgLabelImg.grid(row=1,column=0,sticky=E+W)
        self.dlgXmlButton.grid(row=0,column=1,sticky=E+W)
        self.dlgImgEntry.grid(row=1,column=1,sticky=E+W)
        self.dlgOk.grid(row=2, column=0, sticky=E+W)
        self.dlgCancel.grid(row=2, column=1, sticky=E+W)

        self.dlg.focus_set()
        self.dlg.grab_set()
        self.dlg.transient(self)
        self.dlg.wait_window(self.dlg)

#----------------------------------------------------------------------------------------------------_GetImgPath
#handler needed for preview
  def _GetImgPath(self):
    self.dlg.destroy()
    img = self.dlgImgEntryVar.get()
    if self.XmlToPreview != "EMPTY" and img != "":
        a=VideoTimeLiner(TimeLineWindow, self.evaluator, self.evaluator.inputDataSet.getVideoByName(self.CtrlFile.optionMenuPreview_var.get()), self.evaluator.groundTruthDataSet.getVideoByName(self.CtrlFile.optionMenuPreview_var.get()))
    	a.grid()
        ActreaderWindow = Toplevel()
        ActreaderWindow.geometry("800x600+500+600")
        AcR.External_Call(xml, img, ActreaderWindow)

#----------------------------------------------------------------------------------------------------_GetXml
#handler needed for preview
  def _GetXml(self):
    self.XmlToPreview = tkFileDialog.askopenfilename(parent=self.dlg,defaultextension='.xml', filetypes=[('xml files','*.xml')], initialdir=os.getcwd(), multiple='0', title='Xml file selection')

#----------------------------------------------------------------------------------------------------_CancelPreview
#handler needed for preview
  def _CancelPreview(self):
    self.dlg.destroy()

#----------------------------------------------------------------------------------------------------_ChangeDisplayMode
#launch an additional windows to give a preview of the video and annotation
  def _ChangeDisplayMode(self, event = None):
    #get the files
    Dset = set(self.CtrlFile.DFiles)
    Gset = set(self.CtrlFile.GFiles)
    if len(Dset) == 0 or len (Gset) == 0 :
      return

    if self.CtrlDiag.switchDisplay == "curve":
      self.CtrlDiag.switchDisplay = "matrix"
      self.CtrlFile.buttonMatrix.config(text="Curves")
      self.CtrlFile.buttonProcess.config(state="disable")
      try:
	results = self.evaluator.Confusion_Process(Dset, Gset)
      except hc.harlException as e :
	tkMessageBox.showerror("error", e.cause)
      self.CtrlDiag.CMat = results
      temp = 0
      for i in range(10):
	for j in range(10):
	  temp += self.CtrlDiag.CMat[i][j]
      self.CtrlDiag.nbActDset = temp
    elif self.CtrlDiag.switchDisplay == "matrix":
      self.CtrlDiag.switchDisplay = "Curves"
      self.CtrlFile.buttonMatrix.config(text="Confusion Matrix")
      self.CtrlFile.buttonProcess.config(state="normal")
    self.CtrlDiag.Draw()

#----------------------------------------------------------------------------------------------------_SetSampling
#handling the number of samples for the curve computing
  def _SetSampling(self, event=None):
    if event != None :
      if event.num==4:
	self.CtrlMain.spinSample.invoke('buttonup')
      elif event.num==5:
	self.CtrlMain.spinSample.invoke('buttondown')

    newStep = float(1.0/float(self.CtrlMain.GetSamplesCount()))
    self.CtrlMain.sliderTpt.config(resolution=newStep)
    self.CtrlMain.sliderTrt.config(resolution=newStep)
    self.CtrlMain.sliderTps.config(resolution=newStep)
    self.CtrlMain.sliderTrs.config(resolution=newStep)

#----------------------------------------------------------------------------------------------------_DisplayDFile
#opens a window showing the .xml file selected.
  def _DisplayDFile(self, event=None):
    if self.CtrlFile.listD.curselection() == ():
      return
    xDisp.displayFile(self.CtrlFile.DFiles[int(self.CtrlFile.listD.curselection()[0])], self)

#----------------------------------------------------------------------------------------------------_DisplayGFile
#opens a window showing the .xml file selected.
  def _DisplayGFile(self, event=None):
    if self.CtrlFile.listG.curselection() == ():
      return
    xDisp.displayFile(self.CtrlFile.GFiles[int(self.CtrlFile.listG.curselection()[0])], self)

#----------------------------------------------------------------------------------------------------_ChangeCurveColor_RECALL
#handler attached to the button changing the recall curve color
  def _ChangeCurveColor_RECALL(self, event=None):
    self.ChangeCurveColor("recall", self.CtrlMain.buttonColorTr)

#----------------------------------------------------------------------------------------------------_ChangeCurveColor_PRECISION
#handler attached to the button changing the Precision curve color
  def _ChangeCurveColor_PRECISION(self, event=None):
    self.ChangeCurveColor("precision", self.CtrlMain.buttonColorTp)

#----------------------------------------------------------------------------------------------------_ChangeCurveColor_FSCORE
#handler attached to the button changing the F-Score curve color
  def _ChangeCurveColor_FSCORE(self, event=None):
    self.ChangeCurveColor("fscore", self.CtrlMain.buttonColorFsc)

#----------------------------------------------------------------------------------------------------_Process
#handler attached to the "process button". Launch computing by harlcore.py and display results
  def _Process(self, event=None):
    #get the files
    Dset = set(self.CtrlFile.DFiles)
    Gset = set(self.CtrlFile.GFiles)
    if len(Dset) == 0 or len (Gset) == 0 :
      return
    VarThreshold = self.CtrlMain.SelectedThreshold + ":" + str(int(self.CtrlMain.GetSamplesCount()))
    FixString = ""

    #set up the interface
    self.CtrlFile.buttonProcess.config(state="disable", text="computing")
    self.CtrlFile.buttonProcess.update()
    self.CtrlFile.buttonProcess.update_idletasks()
    self.CtrlDiag.Draw_loading()
    self.CtrlDiag.update()
    self.CtrlFile.buttonProcess.update()
    self.CtrlFile.optionMenuPreview_var.set("")

    #building the fixString
    for i in range(len(THRESHOLDS)):
      FixString += THRESHOLDS[i] + ":" + str(self.CtrlMain.GetFixValue(THRESHOLDS[i]))+":"

    #compute
    try:
      results = self.evaluator.GUI_Process(Dset, Gset, VarThreshold, FixString)
      result_curves = results[0]
      result_perfVal = results[1]
      result_moy = results[2]
      self.CtrlDiag.SetDisplay(self.CtrlMain.SelectedThreshold)
      self.CtrlDiag.SetResults(result_perfVal, result_moy)
      for i in range(len(CURVES)):
	self.CtrlDiag.diagrams[self.CtrlDiag.GetDiagOnScreen()].GetCurves()[CURVES[i]].SetList(result_curves[i])
    except hc.harlException as e :
      tkMessageBox.showerror("error", e.cause)


    #return to the normal interface
    self.CtrlFile.buttonProcess.update()
    self.CtrlFile.buttonProcess.update_idletasks()
    self.CtrlFile.buttonProcess.config(state="normal", text="PROCESS")

    #updating list for video preview
    self.CtrlFile.buttonPreview.configure(state="normal")
    menu = self.CtrlFile.optionMenuPreview["menu"]
    menu.delete(0, "end")
    for string in self.evaluator.inputDataSet.getVideos():
        menu.add_command(label=string,command=lambda value=string:self.CtrlFile.optionMenuPreview_var.set(value))

    #draw the curves
    self.CtrlDiag.UnDraw_loading()
    self.CtrlDiag.Draw()


#====================================================================================================HarlCurve
#storage class for a curve with n points
#at creation you can pass integer as param,it will create an empty list with this size
#you can also give a list and the curve will be initialised with
#it is (really) better to access value trought function even if you can access val[] directly.
class HarlCurve:
#----------------------------------------------------------------------------------------------------__init__
#constructor
  def __init__(self, name, param):
    self.name = name
    self.val = []
    self.size = 0
    if type(param) in {int, long}:
      for i in range(param):
	self.val.append(0)
    elif type(param) in {list}:
      self.size = len(param)
      self.val=param
    else:
      e = "ERROR : bad parameter type - HarlCurve.__init__"
      raise hc.harlException(e)

#----------------------------------------------------------------------------------------------------GetName
  def GetName(self):
    return self.name

#----------------------------------------------------------------------------------------------------GetSize
  def GetSize(self):
    return self.size

#----------------------------------------------------------------------------------------------------GetList
  def GetList(self):
    return self.val

#----------------------------------------------------------------------------------------------------GetValue
  def GetValue(self, index):
    if index in range(len(self.val)):
      return self.val[index]
    else:
      e = "ERROR : out of range - HarlCurve.GetValue."
      raise hc.harlException(e)

#----------------------------------------------------------------------------------------------------SetList
  def SetList(self, newVal):
    if type(newVal) in {list}:
      self.val = newVal
      self.size = len(newVal)
    else:
      e = "ERROR : bad parameter - HarlCurve.SetList"
      raise hc.harlException(e)

#----------------------------------------------------------------------------------------------------SetValue
  def SetValue(self, index, value):
    if index in range(len(self.size)):
      self.val[index]=value
    else:
      e = "ERROR : bad parameter - HarlCurve.SetValue"
      raise hc.harlException(e)

#----------------------------------------------------------------------------------------------------AppendValue
  def AppendValue(self, value):
    self.val.append(value)

#----------------------------------------------------------------------------------------------------RmValue
  def RmValue(self, value):
    if self.val.count(value) != 0:
      self.val.remove(value)
    else:
      e = "ERROR : the curve does't contain the value "+str(value)+" - HarlCurve.RmValue"
      raise hc.harlException(e)

#----------------------------------------------------------------------------------------------------RmAtIndex
  def RmAtIndex(self, index):
    if index in range(self.val):
      self.val.pop(index)
    else:
      e = "ERROR : out of range - HarlCurve.RmAtIndex"
      raise hc.harlException(e)

#====================================================================================================HarlDiagram
#structure for manage 0..n HarlCurves
class HarlDiagram:
#----------------------------------------------------------------------------------------------------__init__
#constructor
  def __init__(self,labelx="X axis", labely="Y axis", title="title" ):
    self.Curves = dict()
    self.LabelX = labelx
    self.LabelY = labely
    self.Title = title


#----------------------------------------------------------------------------------------------------AddCurve
#add a curve to the diagram. If the curve is already in the diagram, this function will raise an harlException.
  def AddCurve(self, curve):
    if curve.GetName() not in self.Curves:
      self.Curves[curve.GetName()] = curve
    else:
      e = "ERROR : A curve in the diagram has already this name : "+str(curve.GetName())+" - HarlDiagram.AddCurve"
      raise hc.harlException(e)

#----------------------------------------------------------------------------------------------------RmCurve
#delete the curve with the name given in parameter from the diagram. If the curve is not in the diagram,
#the function will raise an harlException
  def RmCurve(self, name):
    if name in self.Curves:
      self.Curves.remove(name)
    else:
      e = "ERROR :  There is no curve to delete with this name : "+str(curve.GetName())+" - HarlDiagram.RmCurve"
      raise hc.harlException(e)

  def GetCurves(self):
    return self.Curves

#====================================================================================================HarlCtrlDiagramGUI
#class for manage HarlDiagrams : displays one on n.
class HarlCtrlDiagramGUI(Frame):
#----------------------------------------------------------------------------------------------------__init__
#constructor
  def __init__(self, parent):
    Frame.__init__(self, parent.parent, class_="HarlCtrlDiagramGUI")
    self.parent = parent
    self.diagrams = dict()
    self.margin_L = 30
    self.margin_R = 250
    self.margin_T = 40
    self.margin_B = 30
    self.initUI()
    self.display = None
    self.offsetX=0
    self.offsetY=0
    self.size=0
    self.checksize=0
    self.SetOffSets()
    self.graduation = StringVar()
    self.graduation.set("light scale")
    self.switchDisplay = "curve"


    #GUI varialbes
    self.curveColors = dict()
    for i in range(len(CURVES)):
      self.curveColors[CURVES[i]]=DEFAULT_CURVE_COLORS[i]

    #Data variables
    for i in range(len(THRESHOLDS)):
      self.diagrams[THRESHOLDS[i]]=HarlDiagram(THRESHOLDS[i], "measures",THRESHOLDS[i])
      for j in range(len(CURVES)):
	temp = HarlCurve(CURVES[j], 10)
	self.diagrams[THRESHOLDS[i]].AddCurve(temp)

    self.PV_results = 0
    self.Fsc_results = 0

    self.CMat = None
    self.nbActDset = 0

    self.SetOffSets()
    self.Draw()
#----------------------------------------------------------------------------------------------------initUi
#initialisation of interface widgets
  def initUI(self):
    self.rowconfigure(0, weight=1)
    self.columnconfigure(0, weight=1)
    self.canvas = Canvas(self, bg='#fff',width=400, height=400)
    self.canvas.grid(row=0, column=0, sticky=N+S+E+W)
    self.canvas.bind("<Configure>", self._Resized)

#----------------------------------------------------------------------------------------------------SetOffSets
#function to compute relative size of canvas and drawed elements of the class (for initialisation and resize)
  def SetOffSets(self):
    self.update_idletasks()
    self.checksize=self.winfo_height()*self.winfo_width()
    self.offsetX = 0
    self.offsetY = 0
    H = self.winfo_height() - self.margin_T - self.margin_B
    L = self.winfo_width() - self.margin_L - self.margin_R
    self.size=min(H,L)
    self.x0=self.margin_L
    self.y0=self.margin_T
    self.x1=self.margin_L+self.size
    self.y1=self.margin_T+self.size
    self.margin = 50

#----------------------------------------------------------------------------------------------------Draw
#main drawing function.
#tags used : "resizable", "move", "graduation", "legend", "curve", "loading", "background"
  def Draw(self):
    #background
    self.update_idletasks()
    self.canvas.create_rectangle(0,0, self.winfo_width(), self.winfo_height(),fill="#ffffff",outline="#ffffff",tags=("move", "resizable", "background"))

    if self.switchDisplay == "curve":
      #axis
      self.canvas.create_line(self.x0+self.margin, self.y1-self.margin,self.x1, self.y1-self.margin,arrow="last",tags=("move", "resizable"),width=2)
      self.canvas.create_line(self.x0+self.margin, self.y1-self.margin,self.x0+self.margin, self.y0,arrow="last",tags=("move", "resizable"),width=2)

      #graduation and legend
      self.Draw_graduation()
      self.Draw_legend()

      #curves
      self.canvas.delete("curve")
      if self.display != None and len(self.diagrams[self.display].GetCurves())!=0:
	for i in range(len(CURVES)):
	  self.Draw_curve(self.diagrams[self.display].GetCurves()[CURVES[i]],self.curveColors[CURVES[i]])

    elif self.switchDisplay == "matrix":
      self.canvas.delete("all")
      self.canvas.create_rectangle(0,0, self.winfo_width(), self.winfo_height(),fill="#ffffff",outline="#ffffff",tags=("move", "resizable", "background"))
      self.Draw_ConfMatrix()

    #numerics results
    self.Draw_results(self.PV_results, self.Fsc_results)

    #managment of the "computing" popup
    if len(self.canvas.find_withtag("background")) != 0 and len(self.canvas.find_withtag("loading")) != 0:
      self.canvas.tag_raise("loading","background")

#----------------------------------------------------------------------------------------------------Draw_results
#displays the different numerics results (perfvalue and Fscore average)
  def Draw_results(self, PerfValue, FscoreAvg):
    if PerfValue == 0 or FscoreAvg == 0:
      return

    self.combined_perf = 0.25*(FscoreAvg[0]+FscoreAvg[1]+FscoreAvg[2]+FscoreAvg[3])
    #setting up coords for results
    if (self.canvas.winfo_width()-self.x1) < 200 :
      rx0 = self.x1+0.1*self.margin_R
      rx1 = self.x1+0.95*self.margin_R
    else :
      rx0 = self.x1+0.1*self.margin_R
      rx1 = rx0 + 200
    if (self.y1-self.y0) < 300 :
      ry0 = self.y0
      ry1 = self.y1
    else :
      ry0 = self.y0+(self.y1-self.y0-300)*0.5
      ry1 = ry0+300

    l = rx1-rx0
    h = ry1-ry0

    #drawing the tab
    self.canvas.create_rectangle(rx0,ry0, rx1, ry1, fill="#ffffff",outline="#000", width=2,tags=("result", "background"))

    self.canvas.create_line(rx0,ry0+h/14,rx1,ry0+h/14, tags=("result", "background"))
    self.canvas.create_line(rx0,ry0+2*h/14,rx1,ry0+2*h/14, tags=("result", "background"))
    self.canvas.create_line(rx0,ry0+5*h/14,rx1,ry0+5*h/14, width=2, tags=("result", "background"))
    self.canvas.create_line(rx0,ry0+7*h/14,rx1,ry0+7*h/14, tags=("result", "background"))
    self.canvas.create_line(rx0,ry0+8*h/14,rx1,ry0+8*h/14, tags=("result", "background"))
    self.canvas.create_line(rx0,ry0+12*h/14,rx1,ry0+12*h/14, width=2, tags=("result", "background"))
    self.canvas.create_line(rx0,ry0+13*h/14,rx1,ry0+13*h/14, tags=("result", "background"))

    self.canvas.create_line(rx0+l/2,ry0+h/14,rx0+l/2,ry0+5*h/14, tags=("result", "background"))
    self.canvas.create_line(rx0+l/2,ry0+7*h/14,rx0+l/2,ry0+12*h/14, tags=("result", "background"))

    #titles and labels
    self.canvas.create_text(rx0+l/2,ry0+0.5*h/14, text = "Performance at fixed values", tags=("result"))
    self.canvas.create_text(rx0+l/4,ry0+1.5*h/14, text = "measure", tags=("result"))
    self.canvas.create_text(rx0+3*l/4,ry0+1.5*h/14, text = "value", tags=("result"))
    self.canvas.create_text(rx0+l/4,ry0+2.5*h/14, text = "recall", tags=("result"))
    self.canvas.create_text(rx0+l/4,ry0+3.5*h/14, text = "precision", tags=("result"))
    self.canvas.create_text(rx0+l/4,ry0+4.5*h/14, text = "fscore", tags=("result"))

    self.canvas.create_text(rx0+l/2,ry0+6*h/14, text = "Performance on Fscore (with\nsingle threshold variation)", tags=("result"))#, width=190)
    self.canvas.create_text(rx0+l/4,ry0+7.5*h/14, text = "threshold", tags=("result"))
    self.canvas.create_text(rx0+l/4,ry0+8.5*h/14, text = "trt", tags=("result"))
    self.canvas.create_text(rx0+l/4,ry0+9.5*h/14, text = "tpt", tags=("result"))
    self.canvas.create_text(rx0+l/4,ry0+10.5*h/14, text = "srt", tags=("result"))
    self.canvas.create_text(rx0+l/4,ry0+11.5*h/14, text = "spt", tags=("result"))
    self.canvas.create_text(rx0+3*l/4,ry0+7.5*h/14, text = "value", tags=("result"))

    self.canvas.create_text(rx0+l/2,ry0+12.5*h/14, text = "Combined performance", tags=("result"))

    #filling tabs with values
    self.canvas.create_text(rx0+3*l/4,ry0+2.5*h/14, text = str(PerfValue[0]), tags=("result"))
    self.canvas.create_text(rx0+3*l/4,ry0+3.5*h/14, text = str(PerfValue[1]), tags=("result"))
    self.canvas.create_text(rx0+3*l/4,ry0+4.5*h/14, text = str(PerfValue[2]), tags=("result"))

    self.canvas.create_text(rx0+3*l/4,ry0+8.5*h/14, text = str(FscoreAvg[0]), tags=("result"))
    self.canvas.create_text(rx0+3*l/4,ry0+9.5*h/14, text = str(FscoreAvg[1]), tags=("result"))
    self.canvas.create_text(rx0+3*l/4,ry0+10.5*h/14, text = str(FscoreAvg[2]), tags=("result"))
    self.canvas.create_text(rx0+3*l/4,ry0+11.5*h/14, text = str(FscoreAvg[3]), tags=("result"))

    self.canvas.create_text(rx0+l/2,ry0+13.5*h/14, text = str(self.combined_perf), tags=("result"))



#----------------------------------------------------------------------------------------------------Draw_curve
#draw a curve given in parameter.
  def Draw_curve(self,c,color):
    for i in range(c.GetSize()-1):
      x0=self.margin+self.margin_L+float((self.size-2*self.margin)*i)/float(c.GetSize()-1)
      y0=self.margin+self.margin_T+(1-c.GetValue(i))*(self.size-2*self.margin)
      x1=self.margin+self.margin_L+float((self.size-2*self.margin)*(i+1))/float(c.GetSize()-1)
      y1=self.margin+self.margin_T+(1-c.GetValue(i+1))*(self.size-2*self.margin)
      self.canvas.create_line(x0,y0,x1,y1, fill=color, width=2, tags=("move", "resizable", "curve"))

#----------------------------------------------------------------------------------------------------Draw_ConfMatrix
#draw the confusion matrix for the dataset.
  def Draw_ConfMatrix(self):
    if self.nbActDset == 0:
      return
    size = (self.x1 - self.x0, self.y1 - self.y0)
    self.canvas.create_rectangle(self.x0,self.y0, self.x1, self.y1, fill="#ffffff",outline="#000",width=2, tags="ConfMatrix")
    self.canvas.create_rectangle(self.x0+size[0]*0.2,self.y0+size[1]*0.2, self.x1, self.y1, fill="#ffffff",outline="#000",width=2, tags="ConfMatrix")
    self.canvas.create_line(self.x0, self.y0, self.x0+size[0]*0.2, self.y0+size[1]*0.2)
    self.canvas.create_text(self.x0+size[0]*0.115, self.y0+size[1]*0.015, text="detection", tags="ConfMatrix")
    self.canvas.create_text(self.x0+size[0]*0.085, self.y0+size[1]*0.18, text="ground truth", tags="ConfMatrix")

    if self.CMat != None :
        self.lines_sum = []
        for col in range(10):
            s = 0
            for line in range(10):
                s += self.CMat[line][col]
            self.lines_sum.append(s)

    if self.CMat != None:
        for j in range(10):
    	   for i in range(10):

    	       if i == j and self.lines_sum[i] != 0 :
                   if 255-int(round(255.0*self.CMat[i][j]/float(self.lines_sum[j]))) == 0:
    	               color = "#0000ff"
    	           else:
		      a = str(hex(255-int(round(255.0*self.CMat[i][j]/float(self.lines_sum[j])))))
		      if len(a)>3:
			b = a[(len(a))-2:]
		      else :
			b = "0"+a[(len(a))-1:]
		      color = "#"+b+b+"ff"
               elif self.lines_sum[j] != 0 and 255-int(round(255.0*self.CMat[i][j]/float(self.lines_sum[j]))) == 0:
    	           color = "#ff0000"
               elif self.lines_sum[j] == 0:
                    color = "#ffffff"
               else:
		   a = str(hex(255-int(round(255.0*self.CMat[i][j]/float(self.lines_sum[j])))))
		   if len(a)>3:
		      b = a[(len(a))-2:]
		   else :
		      b = "0"+a[(len(a))-1:]
    	           color = "#ff"+b+b
               self.canvas.create_rectangle(self.x0+size[0]*0.2+((i)/10.0)*float(self.x1-self.x0-size[0]*0.2), self.y0+size[1]*0.2+((j)/10.0)*float(self.y1-self.y0-size[1]*0.2),self.x0+size[0]*0.2+((i+1)/10.0)*float(self.x1-self.x0-size[0]*0.2), self.y0+size[1]*0.2+((j+1)/10.0)*float(self.y1-self.y0-size[1]*0.2), fill=color,width=1, tags="ConfMatrix")
               if self.lines_sum[j] == 0:
                    self.canvas.create_text( self.x0+size[0]*0.2+((i+0.5)/10.0)*float(self.x1-self.x0-size[0]*0.2), self.y0+size[1]*0.2+((j+0.5)/10.0)*float(self.y1-self.y0-size[1]*0.2), text="0.0",  tags="ConfMatrix")
               else:
                    self.canvas.create_text( self.x0+size[0]*0.2+((i+0.5)/10.0)*float(self.x1-self.x0-size[0]*0.2), self.y0+size[1]*0.2+((j+0.5)/10.0)*float(self.y1-self.y0-size[1]*0.2), text=str(round((float(self.CMat[i][j])/float(self.lines_sum[j]))*1000)/10),  tags="ConfMatrix")
    for i in range(10):
      self.canvas.create_line( self.x0+size[0]*0.2+(i/10.0)*float(self.x1-self.x0-size[0]*0.2),self.y0, self.x0+size[0]*0.2+(i/10.0)*float(self.x1-self.x0-size[0]*0.2), self.y1,  tags="ConfMatrix")
      self.canvas.create_line( self.x0,self.y0+size[1]*0.2+(i/10.0)*float(self.y1-self.y0-size[1]*0.2), self.x1, self.y0+size[1]*0.2+(i/10.0)*float(self.y1-self.y0-size[1]*0.2),  tags="ConfMatrix")
      self.canvas.create_text( self.x0+size[0]*0.1, self.y0+size[1]*0.2+((i+0.5)/10.0)*float(self.y1-self.y0-size[1]*0.2), text=ACTION_CLASSES[i],  tags="ConfMatrix")
      self.canvas.create_text( self.x0+size[0]*0.2+((i+0.5)/10.0)*float(self.x1-self.x0-size[0]*0.2), self.y0+size[1]*0.1, text=ACTION_CLASSES[i].replace(' ', '\n'), tags="ConfMatrix" )


#----------------------------------------------------------------------------------------------------Draw_legend
#for each curve, makes an association between the measure and the color curve
  def Draw_legend(self):
    if self.display == None:
      return
    if len(self.diagrams[self.display].GetCurves()) == 0:
      return
    for c in self.diagrams[self.display].GetCurves():
      if c == "recall":
	self.canvas.create_line((self.x1-self.x0)*0.02+self.x0+self.margin, self.y0+self.margin*0.2,(self.x1-self.x0)*0.05+self.x0+self.margin, self.y0+self.margin*0.2,tags=("move", "legend", "resizable"),width=3, fill=self.curveColors[CURVES[0]])
	self.canvas.create_text ( (self.x1-self.x0)*0.06+self.x0+self.margin, self.y0+self.margin*0.2, tags=("move", "resizable", "legend"), text="Recall", anchor="w")
      elif c == "precision":
	self.canvas.create_line((self.x1-self.x0)*0.28+self.x0+self.margin, self.y0+self.margin*0.2,(self.x1-self.x0)*0.31+self.x0+self.margin, self.y0+self.margin*0.2,tags=("move", "legend", "resizable"),width=3, fill=self.curveColors[CURVES[1]])
	self.canvas.create_text ( (self.x1-self.x0)*0.32+self.x0+self.margin, self.y0+self.margin*0.2, tags=("move", "resizable", "legend"), text="Precision", anchor="w")
      elif c == "fscore":
	self.canvas.create_line((self.x1-self.x0)*0.54+self.x0+self.margin, self.y0+self.margin*0.2,(self.x1-self.x0)*0.57+self.x0+self.margin, self.y0+self.margin*0.2,tags=("move", "legend", "resizable"),width=3, fill=self.curveColors[CURVES[2]])
	self.canvas.create_text ( (self.x1-self.x0)*0.58+self.x0+self.margin, self.y0+self.margin*0.2, tags=("move", "resizable", "legend"), text="F-score", anchor="w")

#----------------------------------------------------------------------------------------------------Draw_graduation
#draw the graduation in the graphic, can be none, light or accurate
  def Draw_graduation(self):
    Lgrad=8
    Hgrad=13
    #axis labels
    self.canvas.delete("graduation")
    if self.display != None:
      self.canvas.create_text ((self.x0+self.x1)*0.5, self.y1+(self.margin_B*0.2), tags="graduation", text=self.diagrams[self.display].LabelX)
      self.canvas.create_text (self.x0-(self.margin_L*0.2), (self.y0+self.y1)*0.5, tags="graduation", text=self.diagrams[self.display].LabelY, width=1)

    #axis and graduation
    if self.graduation.get() == "no scale":
      self.canvas.create_line(self.x1-self.margin, self.y1-self.margin+5, self.x1-self.margin, self.y1-self.margin-Hgrad+5, tags="graduation")
      self.canvas.create_line(self.x0+self.margin-5, self.y0+self.margin, self.x0+self.margin+Hgrad-5, self.y0+self.margin, tags="graduation")
    if self.graduation.get() == "accurate scale" or self.graduation.get() == "light scale":
      for i in range(0,11):
	r=float(i)/10.0
	self.canvas.create_line(r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y1-self.margin+5, r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y1-self.margin-Lgrad+5, tags="graduation")
	self.canvas.create_line(self.x0+self.margin+5, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, self.x0+self.margin-Lgrad+5, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, tags="graduation")
	self.canvas.create_text (self.x0+0.5*self.margin, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, tags="graduation", text=str(1-r))
	self.canvas.create_text (r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y1-0.5*self.margin, tags="graduation", text=str(r))
      for i in range(0,3):
	r=float(i)/2.0
	self.canvas.create_line(r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y1-self.margin+5, r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y1-self.margin-Hgrad+5, tags="graduation",width=2)
	self.canvas.create_line(self.x0+self.margin-5, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, self.x0+self.margin+Hgrad-5, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, tags="graduation",width=2)
    if self.graduation.get() == "light scale":
      for i in range(0,11):
	r=float(i)/10.0
	if i%5 ==0:
	  self.canvas.create_line(r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y0+self.margin, r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y1-self.margin, tags="graduation", dash=(8,8))
	  self.canvas.create_line(self.x0+self.margin, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, self.x1-self.margin, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, tags="graduation", dash=(8,8))
    if self.graduation.get() == "accurate scale":
      for i in range(0,11):
	r=float(i)/10.0
	if i%5 !=0:
	  self.canvas.create_line(r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y0+self.margin, r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y1-self.margin, tags="graduation", dash=(3,7))
	  self.canvas.create_line(self.x0+self.margin, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, self.x1-self.margin, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, tags="graduation", dash=(3,7))
	else:
	  self.canvas.create_line(r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y0+self.margin, r*(self.x1-self.x0-2*self.margin)+self.x0+self.margin, self.y1-self.margin, tags="graduation", dash=(8,8),width=2)
	  self.canvas.create_line(self.x0+self.margin, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, self.x1-self.margin, r*(self.y1-self.y0-2*self.margin)+self.y0+self.margin, tags="graduation", dash=(8,8),width=2)
    if len(self.canvas.find_withtag("curve")) != 0 or len(self.canvas.find_withtag("graduation"))!=0:
      self.canvas.tag_raise("curve", "graduation")

#----------------------------------------------------------------------------------------------------Draw_loading
#put a popup on graphic zone when computing is on process
  def Draw_loading(self):
    self.SetOffSets()
    X0=self.x0+0.35*(self.x1-self.x0)
    Y0=self.y0+0.45*(self.y1-self.y0)
    X1=self.x1-0.35*(self.x1-self.x0)
    Y1=self.y1-0.45*(self.y1-self.y0)
    self.canvas.create_rectangle(X0,Y0,X1,Y1,fill="#cccccc",outline="#000000",tags=("move", "loading"))
    self.canvas.create_text ( (X0+X1)*0.5,(Y0+Y1)*0.5, tags=("move", "loading"), text="Compute in progress", anchor="center")
    if len(self.canvas.find_above("loading")) != 0:
      self.canvas.tag_raise("loading",find_above("loading"))

#----------------------------------------------------------------------------------------------------UnDraw_loading
#remove the popup
  def UnDraw_loading(self):
    if len(self.canvas.find_withtag("loading")) != 0 :
      self.canvas.delete("loading")

#----------------------------------------------------------------------------------------------------Draw_onlyDiag
#handler for pdf exportation
  def Draw_onlyDiag(self):
    self.canvas.delete("result")
    self.canvas.scale("all", 0, 0, self.winfo_width()/float(self.x1-self.x0)*0.9, self.winfo_height()/float(self.y1-self.y0)*0.8)

#----------------------------------------------------------------------------------------------------_Resized
#handler for resizing events
  def _Resized(self, event=None):
    if self.checksize != self.winfo_height()*self.winfo_width():
      self.SetOffSets()
      self.Draw()

#----------------------------------------------------------------------------------------------------GetDiagOnScreen
  def GetDiagOnScreen(self):
    return self.display

#----------------------------------------------------------------------------------------------------GetGradStringVar
  def GetGradStrigVar(self):
    return self.graduation

#----------------------------------------------------------------------------------------------------AddDiagram
#add a harldiagram to the instance
  def AddDiagram(self, diag):
    if diag.Title not in self.diagrams:
      self.diagrams[diag.Title]=diag
    else:
      e = "ERROR : A diagram has already this title : "+str(diag.Title)+" - HarlCtrlDiagramGUI.AddDiagram"
      raise hc.harlException(e)

#----------------------------------------------------------------------------------------------------SetDisplay
#set the diagram specified by param title on screen
  def SetDisplay(self,title):
    if title in self.diagrams:
      self.display = title
      self.Draw()

#----------------------------------------------------------------------------------------------------RmDiagram
#delete the diagram specified by param title from instance
  def RmDiagram(self, title):
    if title in self.diagrams:
      self.diagrams.remove(title)
    else:
      e = "ERROR :  There is no diagram to delete with this title : "+str(title)+" - HarlDiagram.RmDiagram"
      raise hc.harlException(e)

#----------------------------------------------------------------------------------------------------SetResults
#set on screen the numeric results from process
  def SetResults(self, PV_res, Fsc_res):
    if type(PV_res) != list or type(Fsc_res) != list:
      return
    if len(PV_res) < 3 or len(Fsc_res) < 4:
      return
    self.PV_results = PV_res
    self.Fsc_results = Fsc_res


#====================================================================================================HarlCtrlMainGUI
#class for user interaction with the core
class HarlCtrlMainGUI(Frame):
#----------------------------------------------------------------------------------------------------__init__
#constructor
  def __init__(self, parent):
    Frame.__init__(self, parent.parent)
    self.parent = parent
    self.initUI()
    self.Thresholds = dict()
    for i in range(len(THRESHOLDS)):
      self.Thresholds[THRESHOLDS[i]]=DEFAULT_THRESHOLDS_VALUE
    self.SelectedThreshold = THRESHOLDS[0]

#----------------------------------------------------------------------------------------------------initUI
#set up the widget interface
  def initUI(self):
    #widgets creation
    self.labelTrt=Label(self, text='Temporal Recall Threshold :')
    self.labelTpt=Label(self, text='Temporal Precision Threshold :')
    self.labelTrs=Label(self, text='Spatial Recall Threshold :')
    self.labelTps=Label(self, text='Spatial Precision Threshold :')
    self.labelRec=Label(self, text='Recall curve color:')
    self.labelPre=Label(self, text='Precision curve color:')
    self.labelFsc=Label(self, text='F-score curve color:')
    self.labelSample=Label(self, text='samples count :')

    self.spinSample=Spinbox(self,values=('10','20','50','100','200','500','1000'),width=4, wrap="true", state="readonly")

    self.Separator01=Frame(self)
    self.Separator02=Frame(self)

    self.sliderVarDict = dict()
    self.vTrt = DoubleVar()
    self.vTrt.set(0.5)
    self.sliderTrt=Scale(self, orient='horizontal',sliderrelief='flat',from_=0.0, to=1.0,resolution=0.1, command=self._SetThreshold, variable=self.vTrt)
    self.sliderVarDict[THRESHOLDS[0]]=self.vTrt
    self.vTpt = DoubleVar()
    self.vTpt.set(0.5)
    self.sliderTpt=Scale(self, orient='horizontal',sliderrelief='flat',from_=0.0, to=1.0,resolution=0.1, command=self._SetThreshold, variable=self.vTpt)
    self.sliderVarDict[THRESHOLDS[1]]=self.vTpt
    self.vTrs = DoubleVar()
    self.vTrs.set(0.5)
    self.sliderTrs=Scale(self, orient='horizontal',sliderrelief='flat',from_=0.0, to=1.0,resolution=0.1, command=self._SetThreshold, variable=self.vTrs)
    self.sliderVarDict[THRESHOLDS[2]]=self.vTrs
    self.vTps = DoubleVar()
    self.vTps.set(0.5)
    self.sliderTps=Scale(self, orient='horizontal',sliderrelief='flat',from_=0.0, to=1.0,resolution=0.1, command=self._SetThreshold, variable=self.vTps)
    self.sliderVarDict[THRESHOLDS[3]]=self.vTps

    self.vGroupRadio = StringVar()
    self.vGroupRadio.set(THRESHOLDS[0])
    self.radioSelec01=Radiobutton(self,variable=self.vGroupRadio, value=THRESHOLDS[0], command=self._ThresholdSelection)
    self.radioSelec02=Radiobutton(self,variable=self.vGroupRadio, value=THRESHOLDS[1], command=self._ThresholdSelection)
    self.radioSelec03=Radiobutton(self,variable=self.vGroupRadio, value=THRESHOLDS[2], command=self._ThresholdSelection)
    self.radioSelec04=Radiobutton(self,variable=self.vGroupRadio, value=THRESHOLDS[3], command=self._ThresholdSelection)

    self.buttonColorTr=Button(self, bg=DEFAULT_CURVE_COLORS[0], activebackground=DEFAULT_CURVE_COLORS[0],relief=FLAT)
    self.buttonColorTp=Button(self, bg=DEFAULT_CURVE_COLORS[1], activebackground=DEFAULT_CURVE_COLORS[1],relief=FLAT)
    self.buttonColorFsc=Button(self, bg=DEFAULT_CURVE_COLORS[2], activebackground=DEFAULT_CURVE_COLORS[2],relief=FLAT)

    #layouts managment
    self.columnconfigure(1, weight=30)
    self.columnconfigure(3, weight=1)
    self.columnconfigure(6, weight=1)

    self.labelTrt.grid(row=0,column=0, sticky=W)
    self.labelTpt.grid(row=1,column=0, sticky=W)
    self.labelTrs.grid(row=2,column=0, sticky=W)
    self.labelTps.grid(row=3,column=0, sticky=W)

    self.sliderTrt.grid(row=0,column=1, sticky=N+S+E+W)
    self.sliderTpt.grid(row=1,column=1, sticky=N+S+E+W)
    self.sliderTrs.grid(row=2,column=1, sticky=N+S+E+W)
    self.sliderTps.grid(row=3,column=1, sticky=N+S+E+W)

    self.radioSelec01.grid(row=0,column=2, sticky=W)
    self.radioSelec02.grid(row=1,column=2, sticky=W)
    self.radioSelec03.grid(row=2,column=2, sticky=W)
    self.radioSelec04.grid(row=3,column=2, sticky=W)

    self.Separator01.grid(row=0,column=3,rowspan=4,sticky=N+S+E+W)
    self.Separator02.grid(row=0,column=6,rowspan=4,sticky=N+S+E+W)

    self.labelRec.grid(row=0,column=4,sticky=W)
    self.labelPre.grid(row=1,column=4,sticky=W)
    self.labelFsc.grid(row=2,column=4,sticky=W)
    self.labelSample.grid(row=3,column=4,sticky=W)

    self.buttonColorTr.grid(row=0,column=5,sticky=W)
    self.buttonColorTp.grid(row=1,column=5,sticky=W)
    self.buttonColorFsc.grid(row=2,column=5,sticky=W)

    self.spinSample.grid(row=3,column=5,sticky=W)

    #binding and events
    self.spinSample.event_add("<<samples>>","<Button-1>","<Button-4>","<Button-5>")
    self.sliderTrt.event_add("<<scroll>>","<Button-4>","<Button-5>")
    self.sliderTpt.event_add("<<scroll>>","<Button-4>","<Button-5>")
    self.sliderTrs.event_add("<<scroll>>","<Button-4>","<Button-5>")
    self.sliderTps.event_add("<<scroll>>","<Button-4>","<Button-5>")

    self.sliderTrt.bind("<<scroll>>", self._SetThreshold)
    self.sliderTpt.bind("<<scroll>>", self._SetThreshold)
    self.sliderTrs.bind("<<scroll>>", self._SetThreshold)
    self.sliderTps.bind("<<scroll>>", self._SetThreshold)

    self.sliderTrt.bind ("<Shift-ButtonRelease-1>", self._SetThreshold_All)
    self.sliderTpt.bind ("<Shift-ButtonRelease-1>", self._SetThreshold_All)
    self.sliderTrs.bind ("<Shift-ButtonRelease-1>", self._SetThreshold_All)
    self.sliderTps.bind ("<Shift-ButtonRelease-1>", self._SetThreshold_All)

    self._ThresholdSelection()

#----------------------------------------------------------------------------------------------------_SetThreshold_All
#get the slider thresholds values
  def _SetThreshold_All(self, event=None):
    if event != None :
      for key in self.sliderVarDict:
	self.sliderVarDict[key].set(event.widget.get())
	self.Thresholds[key] = self.sliderVarDict[key].get()
#----------------------------------------------------------------------------------------------------GetSamplesCount
  def GetSamplesCount(self):
    return self.spinSample.get()

#----------------------------------------------------------------------------------------------------GetFixValue
  def GetFixValue(self, threshold):
    if threshold in self.Thresholds:
      return self.Thresholds[threshold]

#----------------------------------------------------------------------------------------------------_SetThreshold
#set the value of a threshold
  def _SetThreshold(self, event=None):
    if event != None and type(event) != str:
      if event.num==4:
	event.widget.set(event.widget.get()+1.0/float(self.spinSample.get()))
      elif event.num==5:
	event.widget.set(event.widget.get()-1.0/float(self.spinSample.get()))

    if self.vTrt.get() != self.Thresholds['trt']:
      self.Thresholds['trt'] = self.vTrt.get()
    if  self.vTpt.get() != self.Thresholds['tpt']:
      self.Thresholds['tpt'] = self.vTpt.get()
    if self.vTrs.get() != self.Thresholds['srt']:
      self.Thresholds['srt'] = self.vTrs.get()
    if self.vTps.get() != self.Thresholds['spt']:
      self.Thresholds['spt'] = self.vTps.get()

#----------------------------------------------------------------------------------------------------_ThresholdSelection
#set the selected threshold (the fixed one for process)
  def _ThresholdSelection(self, event = None):
    if self.vGroupRadio.get() ==THRESHOLDS[0]:
      self.SelectedThreshold = THRESHOLDS[0]
      self.sliderTrt.config( troughcolor='#606060')
      self.sliderTpt.config( troughcolor='#a0a0a0')
      self.sliderTrs.config( troughcolor='#a0a0a0')
      self.sliderTps.config( troughcolor='#a0a0a0')
    elif self.vGroupRadio.get() == THRESHOLDS[1]:
      self.SelectedThreshold = THRESHOLDS[1]
      self.sliderTrt.config( troughcolor='#a0a0a0')
      self.sliderTpt.config( troughcolor='#606060')
      self.sliderTrs.config( troughcolor='#a0a0a0')
      self.sliderTps.config( troughcolor='#a0a0a0')
    elif self.vGroupRadio.get() == THRESHOLDS[2]:
      self.SelectedThreshold = THRESHOLDS[2]
      self.sliderTrt.config( troughcolor='#a0a0a0')
      self.sliderTpt.config( troughcolor='#a0a0a0')
      self.sliderTrs.config( troughcolor='#606060')
      self.sliderTps.config( troughcolor='#a0a0a0')
    elif self.vGroupRadio.get() == THRESHOLDS[3]:
      self.SelectedThreshold = THRESHOLDS[3]
      self.sliderTrt.config( troughcolor='#a0a0a0')
      self.sliderTpt.config( troughcolor='#a0a0a0')
      self.sliderTrs.config( troughcolor='#a0a0a0')
      self.sliderTps.config( troughcolor='#606060')


#====================================================================================================HarlCtrlFileGUI
#widget managing files in the application
class HarlCtrlFileGUI(Frame):
#----------------------------------------------------------------------------------------------------__init__
#constructor
  def __init__(self, parent):
    Frame.__init__(self, parent.parent)
    self.parent = parent
    self.DFiles = []
    self.Dvar=StringVar()
    self.GFiles = []
    self.Gvar=StringVar()
    self.initUI()

#----------------------------------------------------------------------------------------------------initUI
#Set up the widget interface
  def initUI(self):
    #widgets creation
    self.buttonProcess = Button(self,text="PROCESS", state="disable")
    self.buttonMatrix = Button(self, text="Confusion Matrix", command=self.parent._ChangeDisplayMode, state="disable")
    self.optionMenuPreview_var = StringVar()
    self.optionMenuPreview_var.set("")
    self.optionMenuPreview = OptionMenu(self, self.optionMenuPreview_var, [])
    self.buttonPreview = Button(self, text='preview',command=self.parent._Preview,state="disable")
    self.buttonDAdd=Button(self, text='Add', command=self._AddDFile)
    self.buttonDDel=Button(self, text='Delete', command=self._DelDFile)
    self.buttonDRst=Button(self, text='Reset', command=self._ResetDFiles)
    self.buttonDDisp=Button(self, text='Display', command=self.parent._DisplayDFile)
    self.buttonGAdd=Button(self, text='Add', command=self._AddGFile)
    self.buttonGDel=Button(self, text='Delete', command=self._DelGFile)
    self.buttonGRst=Button(self, text='Reset', command=self._ResetGFiles)
    self.buttonGDisp=Button(self, text='Display', command=self.parent._DisplayGFile)

    self.labelDFiles=Label(self, text='Detection Files :')
    self.labelGFiles=Label(self, text='Ground Truth Files : ')

    self.Separator01=Frame(self)
    self.Separator02=Frame(self)
    self.Separator03=Frame(self)
    self.Separator04=Frame(self)

    self.listD=Listbox(self, listvariable =self.Dvar, selectmode="single", exportselection=0 )
    self.listG=Listbox(self, listvariable =self.Gvar, selectmode="single", exportselection=0 )

    #Pannel-level layouts managment
    self.rowconfigure(0, weight=1)
    self.rowconfigure(2, weight=1)
    self.rowconfigure(4, weight=10)
    self.rowconfigure(7, weight=1)
    self.rowconfigure(9, weight=10)
    self.rowconfigure(12, weight=1)

    self.Separator01.grid(row=0,column=0,columnspan=3,sticky=N+S+E+W)
    self.optionMenuPreview.grid(row=1, column=0, columnspan=2, sticky=N+S+E+W)
    self.buttonPreview.grid(row=1, column=2, sticky=E+W)
    self.Separator02.grid(row=2,column=0,columnspan=3,sticky=N+S+E+W)
    self.labelDFiles.grid(row=3, column=0, columnspan=3, sticky=E+W)
    self.listD.grid(row=4, column=0, columnspan=3, sticky=N+E+W+S)
    self.buttonDAdd.grid(row=5, column=0, sticky=E+W)
    self.buttonDDel.grid(row=5, column=1, sticky=E+W)
    self.buttonDRst.grid(row=5, column=2, sticky=E+W)
    self.buttonDDisp.grid(row=6, column=0, columnspan=3, sticky=E+W)
    self.Separator03.grid(row=7,column=0,columnspan=3,sticky=N+S+E+W)
    self.labelGFiles.grid(row=8, column=0, columnspan=3, sticky=E+W)
    self.listG.grid(row=9, column=0, columnspan=3, sticky=N+E+W+S)
    self.buttonGAdd.grid(row=10, column=0, sticky=E+W)
    self.buttonGDel.grid(row=10, column=1, sticky=E+W)
    self.buttonGRst.grid(row=10, column=2, sticky=E+W)
    self.buttonGDisp.grid(row=11, column=0, columnspan=3, sticky=E+W)
    self.Separator04.grid(row=12, column=0,columnspan=3,sticky=N+S+E+W)
    self.buttonMatrix.grid(row=13, column=0, columnspan=3, sticky=E+W)
    self.buttonProcess.grid(row=14, column=0, columnspan=3, sticky=E+W)

#----------------------------------------------------------------------------------------------------_AddDFile
#add a file to detection file list
  def _AddDFile(self, event = None):
    result = tkFileDialog.askopenfilename(parent=self.buttonDAdd,defaultextension='.xml', filetypes=[('xml files','*.xml')], initialdir=os.getcwd(), multiple='1', title='Detection file seclection')
    #converting result for windows from unicode char to tab[str] correctly formated
    if type(result) == unicode:
      result = str(result)
      result = result.split('} {')
      tab = []
      for i in range(len(result)):
	result[i] = result[i].strip('{').strip('}')
    for r in result:
      if r not in self.DFiles:
	self.DFiles.append(r)
    temp = ""
    for f in self.DFiles:
      temp +=f.split('/')[len(f.split('/'))-1]+" "
    self.Dvar.set(temp)
    #gui buttons state managment
    if len(self.DFiles)!=0 and len(self.GFiles)!=0 :
      self.buttonProcess.config(state="normal")
      self.buttonMatrix.config(state="normal")
      self.buttonProcess.update()
      self.buttonProcess.update_idletasks()

#----------------------------------------------------------------------------------------------------_AddGFile
#add a file to the ground truth file list
  def _AddGFile(self, event = None):
    result = tkFileDialog.askopenfilename(parent=self.buttonGAdd,defaultextension='.xml', filetypes=[('xml files','*.xml')], initialdir=os.getcwd(), multiple='1', title='Ground Truth file seclection')
    #converting result for windows from unicode char to tab[str] correctly formated
    if type(result) == unicode:
      result = str(result)
      result = result.split('} {')
      tab = []
      for i in range(len(result)):
	result[i] = result[i].strip('{').strip('}')
    for r in result:
      if r not in self.GFiles:
	self.GFiles.append(r)
    temp = ""
    for f in self.GFiles:
      temp +=f.split('/')[len(f.split('/'))-1]+" "
    self.Gvar.set(temp)
    #gui buttons state managment
    if len(self.DFiles)!=0 and len(self.GFiles)!=0 :
      self.buttonProcess.config(state="normal")
      self.buttonMatrix.config(state="normal")
      self.buttonProcess.update()
      self.buttonProcess.update_idletasks()

#----------------------------------------------------------------------------------------------------_DelDFile
#delete the selected file from detection file list
  def _DelDFile(self, event = None):
    if self.listD.curselection() ==():
      return
    temp = self.listD.curselection()[0]
    self.listD.delete(int(temp))
    self.DFiles.pop(int(temp))
    #gui buttons state managment
    if len(self.DFiles)==0 or len(self.GFiles)==0 :
      self.buttonProcess.config(state="disable")
      self.buttonMatrix.config(state="disable")

#----------------------------------------------------------------------------------------------------_DelGFile
#delete the selected file from the ground truth file list
  def _DelGFile(self, event = None):
    if self.listG.curselection() ==():
      return
    temp = self.listG.curselection()[0]
    self.listG.delete(int(temp))
    self.GFiles.pop(int(temp))
    #gui buttons state managment
    if len(self.DFiles)==0 or len(self.GFiles)==0 :
      self.buttonProcess.config(state="disable")
      self.buttonMatrix.config(state="disable")

#----------------------------------------------------------------------------------------------------_ResetDFiles
#reset the detection file list
  def _ResetDFiles(self, event = None):
    self.Dvar.set("")
    self.DFiles = []
    #gui buttons state managment
    self.buttonProcess.config(state="disable")
    self.buttonMatrix.config(state="disable")

#----------------------------------------------------------------------------------------------------_ResetGFiles
#reset the grund truth file list
  def _ResetGFiles(self, event = None):
    self.Gvar.set("")
    self.GFiles = []
    self.buttonProcess.config(state="disable")
    self.buttonMatrix.config(state="disable")

#----------------------------------------------------------------------------------------------------_ResetFiles
#reset both file lists
  def _ResetFiles(self, event = None):
    self._ResetDFiles()
    self._ResetGFiles()


#====================================================================================================VideoTimeLiner
#class to display in a new window the time line matching representation of a video
class VideoTimeLiner(Frame):
#----------------------------------------------------------------------------------------------------__init__
#constructor
  def __init__(self, parent, evaluator, Dvideo, Gvideo):
    if not isinstance(Dvideo, hc.EvVideo) or not isinstance(Gvideo, hc.EvVideo):
	e = "ERROR in constructor of VideoTimeLiner : args are not instances of EvVideo"
	raise hc.harlException(e)
    if Dvideo.name != Gvideo.name :
        e = "ERROR in constructor of VideoTimeLiner : videos have not the same name..."
        raise hc.harlException(e)
    if not isinstance(evaluator, hc.Evaluator):
        e = "ERROR in constructor of VideoTimeLiner : the evaluator is not an instance of Evaluator"
        raise hc.harlException(e)

    Frame.__init__(self, parent)
    self.evaluator = evaluator
    self.parent = parent
    self.Dvid = Dvideo
    self.Gvid = Gvideo
    self.ActCount = max(len(Dvideo.getActions()), len(Gvideo.getActions()))
    self.DispCount = len(self.Dvid.getActions())
    self.firstFrame = -1
    self.lastFrame = -1
    self.graduation = 10

    self.Process()

    self.initUI()

#----------------------------------------------------------------------------------------------------Process
#main processing function of the class
  def Process(self):
    self.matching = self.evaluator.VideoComp_Process(self.Dvid.name)
    for i in range(self.DispCount):
        if self.firstFrame > self.Dvid.getActions()[i].getRect()[0].actId or self.firstFrame == -1:
            self.firstFrame = self.Dvid.getActions()[i].getRect()[0].actId
        if self.lastFrame < self.Dvid.getActions()[i].getRect()[len(self.Dvid.getActions()[i].getRect())-1].actId :
            self.lastFrame = self.Dvid.getActions()[i].getRect()[len(self.Dvid.getActions()[i].getRect())-1].actId
    for i in range(len(self.Gvid.getActions())):
        if self.firstFrame > self.Gvid.getActions()[i].getRect()[0].actId :
            self.firstFrame = self.Gvid.getActions()[i].getRect()[0].actId
        if self.lastFrame < self.Gvid.getActions()[i].getRect()[len(self.Gvid.getActions()[i].getRect())-1].actId :
            self.lastFrame = self.Gvid.getActions()[i].getRect()[len(self.Gvid.getActions()[i].getRect())-1].actId
    if self.firstFrame < round(0.1*(self.lastFrame - self.firstFrame)) :
        self.firstFrame = 0
    else:
        self.firstFrame -= int(round(0.1*(self.lastFrame - self.firstFrame)))
    self.lastFrame += int(round(0.1*(self.lastFrame - self.firstFrame)))

    self.graduation = 10+(int(self.lastFrame-self.firstFrame)/400)*10

#----------------------------------------------------------------------------------------------------initUI
#set up the widget interface
  def initUI(self):
    self.setOffsets()
    self.columnconfigure(0,weight=1)
    self.rowconfigure(0,weight=1)

    self.scroll = Scrollbar(self,orient="vertical",width=15)
    self.canvas = Canvas(self, bg='#dddddd',width=980, height=200, yscrollcommand=self.scroll.set, scrollregion=(0,0,980,(50*self.DispCount)+self.header+self.footer), confine="true")
    self.scroll.configure(command=self.canvas.yview)
    self.canvas.grid(row=0, column=0, sticky=N+S+E+W)
    self.scroll.grid(row=0, column=1, sticky=N+S)
    self.canvas.bind("<Configure>", self._Resized)
    self.setOffsets()
    self.Draw()

#----------------------------------------------------------------------------------------------------setOffsets
#on resize calculate the new internal drawing dimentions
  def setOffsets(self):
    self.update_idletasks()
    self.header = 40
    self.footer = 40
    self.legend = 100
    self.x0 = self.legend
    self.x1 = self.winfo_width()-15

    self.y0 = self.header
    self.y1 = (50*self.DispCount)+self.header

#----------------------------------------------------------------------------------------------------Draw
#main drawing function
  def Draw(self):
    self.canvas.delete("all")

    #don't draw during initialisation
    if(self.x1-self.x0)<2:
        return

    h = self.y1 - self.y0
    l = self.x1 - self.x0
    w = l/float(self.lastFrame - self.firstFrame+1)
    notFounded = 0

    self.canvas.create_rectangle(self.x0, self.y0, self.x1, self.y1, fill="#ffffff", width=0, tags="background")
    self.canvas.create_line(0, self.y0, self.x1, self.y0, fill="#000", tags="lines", width=2)
    self.canvas.create_line(self.x0, 0, self.x0, self.y1, fill="#000", tags="lines", width=2)

    for i in range(self.DispCount):
        y = int(self.y0+(i+1)*h/float(self.DispCount))

    #timelines legend
        self.canvas.create_line(self.x0, y-10, self.x1, y-10, fill="#000", tags="lines")
        self.canvas.create_line(0, y, self.x1, y, fill="#000", tags="lines", width=2)
        self.canvas.create_line(self.x0, y-40, self.x1, y-40, fill="#000", tags="lines")

        self.canvas.create_text(self.legend/2, y-25, text=("action n°"+str(i)))
        self.canvas.create_text(self.legend/2, y-7, text=("Ground Truth"))
        self.canvas.create_text(self.legend/2, y-43, text=("Detection"))

    #if the detection match no action from grund truth
        if (i-notFounded) >= len(self.matching) or self.matching[i-notFounded][0] != self.Dvid.getActions()[i] :
            notFounded += 1
            self.canvas.create_text(self.x0 + int(0.5*(self.x1-self.x0)), y-20, text=("NO MATCHING AVIABLE."))

    #drawing the graphics
        else :
            actGt = self.matching[i-notFounded][1]
            actDt = self.matching[i-notFounded][0]

            for j in range(self.lastFrame - self.firstFrame+1):
                x=(self.x0+(j+0.5)*w)

                if (j+self.firstFrame) % self.graduation == 0:
                    self.canvas.create_line(x, y, x, y-50, fill="#404040", width=1, tags="background")
                    if (i-notFounded) == 0:
                        self.canvas.create_text(x, self.header*0.75, text=(str(j+self.firstFrame)))
                        self.canvas.create_line(x, self.y0, x, self.y0-5, fill="#404040", width=1, tags="background")


                if (j+self.firstFrame) in range(actGt.getRect()[0].actId, actGt.getRect()[len(actGt.getRect())-1].actId):
                    test = 1
                    self.canvas.create_line(x, y, x, y-10, fill="#909090", width=w*1.1, tags="colors")
                    self.canvas.create_line(x, y-39, x, y-10, fill="#e0a0a0", width=w*1.1, tags="colors")
                else:
                    test = 0

                if (j+self.firstFrame) in range(actDt.getRect()[0].actId, actDt.getRect()[len(actDt.getRect())-1].actId):
                    self.canvas.create_line(x, y-40, x, y-50, fill="#909090", width=w*1.1, tags="colors")
                    self.canvas.create_line(x, y-39, x, y-10, fill="#e0a0a0", width=w*1.1, tags="colors")
                    if test == 1:
                        a = actGt.getRect()[j-actGt.getRect()[0].actId+self.firstFrame]
                        b = actDt.getRect()[j-actDt.getRect()[0].actId+self.firstFrame]
                        overlap=float((min(a.x2, b.x2)-max(a.x1, b.x1))*(min(a.y2, b.y2)-max(a.y1, b.y1)))
                        recall = overlap/float((a.x2-a.x1)*(a.y2-a.y1))
                        precision = overlap/float((b.x2-b.x1)*(b.y2-b.y1))
                        frameFScore = 2*precision*recall/(precision+recall)
                        self.canvas.create_line(x, y-39, x, y-10, fill="#a0a0e0", width=w*1.1, tags="colors")
                    else:
                        frameFScore = 0
                    color = "#20e020"
                    self.canvas.create_line(x, y-int(29*frameFScore)-10, x, y-10, fill=color, width=w*1.1, tags="colors")
    #Drawing header
    self.canvas.create_text(self.x1*0.5, self.header*0.3, text=("Time line representation of video : "+self.Dvid.name))
    self.canvas.create_text(self.x0*0.5, self.header*0.75, text=("Actions detected :"))
    self.canvas.create_text(self.x0+5, self.header*0.45, text=("Frames ["+str(self.firstFrame)+"-"+str(self.lastFrame)+"] : "), anchor=W)
    #Drawing footer
    n = len(self.Gvid.getActions())-(self.DispCount - notFounded)
    if n != 0 :
        self.canvas.create_text(self.x1/2, self.y1+self.footer*0.5, text=(str(n)+" actions in ground truth with no match in detection." ))
    else :
        self.canvas.create_text(self.x1/2, self.y1+self.footer*0.5, text="All actions in ground truth matched.")
    #sorting elements on z-axis
    self.canvas.tag_raise("colors")
    self.canvas.tag_raise("lines")

#----------------------------------------------------------------------------------------------------_Resized
#handler for windows size
  def _Resized(self, event = None):
    self.update_idletasks()
    self.setOffsets()
    self.Draw()




#----------------------------------------------------------------------------------------------------main
#main function
def main():
  path = os.getcwd()
  root = Tk()
  global mainFrame
  mainFrame = HarlGUI(root, path)
  root.mainloop()

if __name__ == '__main__':
  main()
